package com.test.human.api;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.transaction.UserTransaction;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.base.MapGlobalResolver;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.compiler.BPMN2ProcessFactory;
import org.drools.compiler.ProcessBuilderFactory;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.marshalling.impl.ProcessMarshallerFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessRuntimeFactory;
import org.h2.tools.DeleteDbFiles;
import org.h2.tools.Server;
import org.jbpm.bpmn2.BPMN2ProcessProviderImpl;
import org.jbpm.marshalling.impl.ProcessMarshallerFactoryServiceImpl;
import org.jbpm.process.builder.ProcessBuilderFactoryServiceImpl;
import org.jbpm.process.instance.ProcessRuntimeFactoryServiceImpl;
import org.jbpm.process.workitem.wsht.WSHumanTaskHandler;
import org.jbpm.task.query.TaskSummary;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;
/**
 * This is a example that shows JBPM5 API usage.
 */
public class ProcessTaskAPITest {

	
	
	public static final void main(String[] args) {
		/*
		 * Start local h2 datbase
		 * This is not required if the application connects to a remote database
		 */
	
		try {
			DeleteDbFiles.execute("", "JPADroolsFlow", true);
			 Server h2Server = Server.createTcpServer(new String[0]);
			h2Server.start();
		} catch (SQLException e) {
			throw new RuntimeException("can't start h2 server db",e);
		}
	  
		
		try {
			
			UserTransaction ut = (UserTransaction) new InitialContext().lookup( "java:comp/UserTransaction" );
            ut.begin();

			PoolingDataSource ds1 = new PoolingDataSource();
	       
			
			//sample H2 database configuration
			
			
			ds1.setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
	    	ds1.setUniqueName("jdbc/testDS1");
	    	ds1.setMaxPoolSize(5);
	    	ds1.setAllowLocalTransactions(true);
	    	ds1.getDriverProperties().setProperty("driverClassName", "org.h2.Driver");
	    	ds1.getDriverProperties().setProperty("url", "jdbc:h2:tcp://localhost/JPADroolsFlow");
	    	ds1.getDriverProperties().setProperty("user", "sa");
	    	ds1.getDriverProperties().setProperty("password", "");
	    	
			
	    	
			//Sample Oracle datasource configuration
	    	/*
			ds1.setUniqueName( "jdbc/testDS1" );
	        ds1.setClassName( "oracle.jdbc.xa.client.OracleXADataSource" );
	        ds1.setMaxPoolSize( 3 );
	        ds1.setAllowLocalTransactions( true );
	        ds1.getDriverProperties().put( "user","user" );
	        ds1.getDriverProperties().put( "password","pw" );
	        ds1.getDriverProperties().put( "URL",
	        "jdbc:oracle:thin:@oracleserver:1521:db" );
			*/
			//Sample Mysql datasource configuration
	    	/*
			   ds1 = new PoolingDataSource();
		       ds1.setUniqueName( "jdbc/testDS1" );
		       ds1.setClassName( "com.mysql.jdbc.jdbc2.optional.MysqlXADataSource" );
		        
		       ds1.setMaxPoolSize( 3 );
		       ds1.setAllowLocalTransactions( true );
		       ds1.getDriverProperties().put( "user","user" );
		       ds1.getDriverProperties().put( "password","user" );
		       ds1.getDriverProperties().put( "URL",
		                                     "jdbc:mysql://mysqlserver:3306/jbpm5db" );
             */
		        
	        ds1.init();
	        
	        
			// load up the knowledge base
	        KnowledgeBase kbase = readKnowledgeBase("HelloProcess.bpmn");
			StatefulKnowledgeSession ksession = createKnowledgeSession(kbase);
			
	        /*Create the knowledge base for HelloProcss.bpmn */
	       
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(ksession, "testlog", 1000);
			ksession.getWorkItemManager().registerWorkItemHandler("Human Task", new WSHumanTaskHandler());
			//JPAWorkingMemoryDbLogger dblogger = new JPAWorkingMemoryDbLogger(ksession);
			
			System.out.println(" started the humantask");
			
			
			
			ksession.startProcess("Hello");	
			ksession.fireAllRules();
			ut.commit();
			
            ut.begin();		
			
			/*
			 * Get all the task assigned to 'krisv'
			 */
            
			List <TaskSummary> tasks = JbpmAPIUtil.getAssignedTasks("krisv");
			//TaskSummary task = null;  
			
		
			//BlockingTaskOperationResponseHandler responseHandler = null;
			//ContentData contentData = null;
			Map<String,String> data = new HashMap<String,String>();
			
			for (TaskSummary taskSummary : tasks) {   
			System.out.println(taskSummary.getId() + " : "     + taskSummary.getName()); 
			
			/*
			 * Set the data that needs to be processed by the task.
			 * This data is typically set in web form 
			 */
			data.put("priority","High");
			data.put("modelNumber","1234");
			data.put("quantity","225");
			
			/*
			 * Complete the task using API
			 */
			JbpmAPIUtil.completeTask(taskSummary.getId(), data, "krisv");
		  }  
			
			System.out.println("****finished krisv");
			ut.commit();
			logger.close();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	
	/*
	 * Load the bpmn file into knowledgebase
	 */
	private static KnowledgeBase readKnowledgeBase(String process) throws Exception {
		ProcessBuilderFactory.setProcessBuilderFactoryService(new ProcessBuilderFactoryServiceImpl());
		ProcessMarshallerFactory.setProcessMarshallerFactoryService(new ProcessMarshallerFactoryServiceImpl());
		ProcessRuntimeFactory.setProcessRuntimeFactoryService(new ProcessRuntimeFactoryServiceImpl());
		BPMN2ProcessFactory.setBPMN2ProcessProvider(new BPMN2ProcessProviderImpl());
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource(process), ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}

	
	
	/*
	 * Create EntityManagerFactory and register it in the environment
	 * Create the knowledge session that uses JPA to persists runtime state
	 */
	
	private static StatefulKnowledgeSession createKnowledgeSession(KnowledgeBase kbase) {
		EntityManagerFactory emf = Persistence.createEntityManagerFactory( "org.jbpm.persistence.jpa" );
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set( EnvironmentName.ENTITY_MANAGER_FACTORY, emf );
		env.set( EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager() );
		env.set( EnvironmentName.GLOBALS, new MapGlobalResolver() );
		
		Properties properties = new Properties();
		properties.put("drools.processInstanceManagerFactory", "org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
		properties.put("drools.processSignalManagerFactory", "org.jbpm.persistence.processinstance.JPASignalManagerFactory");
		KnowledgeSessionConfiguration config = KnowledgeBaseFactory.newKnowledgeSessionConfiguration(properties);
	
		return JPAKnowledgeService.newStatefulKnowledgeSession(kbase, config, env);
			
	}
}
